#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

void empty_output(){
  fputs("No output\n", stderr);
  puts("0.0\n");
  exit(0);
}

void present_err() {
  fputs("Presentation error\n", stderr);
  puts("0.0\n");
  exit(0);
}

void wrong_ans() {
  fputs("Output isn't correct\n", stderr);
  puts("0.0\n");
  exit(0);
}

void correct_ans() {
  fputs("Output is correct\n", stderr);
  puts("1.0\n");
  exit(0);
}

void partial_ans() {
  fputs("Output is correct, not longest\n", stderr);
  puts("0.2\n");
  exit(0);
}

int safe_to_int(char *s) {
  int len = strlen(s);
  if (len <= 0 || len > 9) present_err();  // Wrong size
  for (int i = 0; i < len; ++i) {
    if (!isdigit(s[i])) present_err();  // Contains carbage
  }
  return atoi(s);
}

int minrot(char *s, int n) {  // O(N) worst case
  int a = 0;
  for (int b = 0; b < n; ++b) {
    for (int i = 0; i < n; ++i) {
      if (a + i == b || s[(a + i) % n] < s[(b + i) % n]) {
        if (i > 1) b += i - 1;
        break;
      }
      if (s[(a + i) % n] > s[(b + i) % n]) {
        a = b;
        break;
      }
    }
  }
  return a;
}

int main(int argc, char **argv) {
  if (argc != 4) {
    puts("Usage: checker <input file> <true output file> <student's output file>\n");
    return 0;
  }
  FILE *inp = fopen(argv[1], "r");
  FILE *ans = fopen(argv[2], "r");
  FILE *out = fopen(argv[3], "r");
  if(!out)
    empty_output();

  char *whitespace = " \t\v\r\n\f";

  int ans_len, ans_loc1, ans_loc2;
  fscanf(ans, "%d %d %d", &ans_len, &ans_loc1, &ans_loc2);

  const int max_out_len = 100;
  char out_line1[max_out_len], out_line2[max_out_len];
  if (!fgets(out_line1, max_out_len, out)) present_err();  // File ended early
  char *out_len_str = strtok(out_line1, whitespace);
  if(!out_len_str) present_err(); // No length
  int out_len = safe_to_int(out_len_str);
  if (out_len <= 0)
    present_err();  // non-positive length
  if (strtok(NULL, whitespace)) present_err();  // first line contained extra values
  if (!fgets(out_line2, max_out_len, out)) present_err();  // File ended early
  char *token1 = strtok(out_line2, whitespace);
  char *token2 = strtok(NULL, whitespace);
  if(!token2) present_err(); //second line didn't contain two words
  int out_loc[2];
  out_loc[0] = safe_to_int(token1);
  out_loc[1] = safe_to_int(token2);
  if (strtok(NULL, whitespace)) present_err();  // second line contained extra values

  if (out_len > ans_len || out_len * 2 < ans_len) wrong_ans();

  const int maxn = 1e5;
  char *s[2];
  int n[2];
  for (int i = 0; i < 2; ++i) {
    s[i] = (char *)malloc(maxn);
    fscanf(inp, "%s", s[i]);
    n[i] = strlen(s[i]);
  }

  for (int i = 0; i < 2; ++i) {
    if (out_loc[i] < 0 || out_loc[i] + out_len > n[i]) wrong_ans();
  }

  char *nl[2];
  for (int i = 0; i < 2; ++i) {
    nl[i] = s[i] + out_loc[i];
  }
  int correct_rot_found = 0;
  for (int rot = 0; rot < 2; ++rot) {
    int nl_loc[2];
    for (int i = 0; i < 2; ++i) {
      nl_loc[i] = minrot(nl[i], out_len);
    }
    int i = 0;
    for (; i < out_len; ++i) {
      if (nl[0][(nl_loc[0] + i) % out_len] != nl[1][(nl_loc[1] + i) % out_len]) {
        break;
      }
    }
    if (i == out_len) {
      correct_rot_found = 1;
      break;
    }
    if (!rot) {
      int l = 0, r = out_len;
      while (l < r) {
        --r;
        char tmp = nl[1][l];
        nl[1][l] = nl[1][r];
        nl[1][r] = tmp;
        ++l;
      }
    }
  }
  if (!correct_rot_found) {
    wrong_ans();
  }

  if (out_len == ans_len) {
    correct_ans();
  }
  partial_ans();
}
