#include <algorithm>
#include <cassert>
#include <chrono>
#include <fstream>
#include <iostream>
#include <random>
using namespace std;
typedef long long ll;

mt19937_64 gen;

ofstream GEN_file;
ofstream desc_file;

vector< string > gen_hard(string atom, int avg_atom_cnt, vector< string > tokens, int max_len) {
  uniform_int_distribution< int > atom_cnt_distr(1, avg_atom_cnt - 1);
  vector< vector< string > > token_ord = {tokens, tokens};
  bool done = false;
  int sz = tokens.size();
  while (!done) {
    for (int i = 0; i < 2; ++i) {
      shuffle(token_ord[i].begin(), token_ord[i].end(), gen);
    }
    done = true;
    for (int i = 0; i < sz; ++i) {
      for (int j = 0; j < sz; ++j) {
        if (token_ord[0][i] == token_ord[1][j] &&
            token_ord[0][(i + 1) % sz] == token_ord[1][(j + 1) % sz]) {
          done = false;
        }
      }
    }
  }
  vector< string > res(2);
  for (int i = 0; i < 2; ++i) {
    int cnt = atom_cnt_distr(gen) + atom_cnt_distr(gen);
    for (int j = 0; j < cnt; ++j) {
      res[i] += atom;
    }
  }
  for (int i = 0; i < 2; ++i) {
    for (int j = 0;; j = (j + 1) % sz) {
      int cnt = atom_cnt_distr(gen) + atom_cnt_distr(gen);
      if ((int)res[i].size() + cnt * (int)atom.size() + (int)token_ord[i][j].size() > max_len)
        break;
      res[i] += token_ord[i][j];
      for (int j = 0; j < cnt; ++j) {
        res[i] += atom;
      }
    }
  }
  for (int i = 0; i < 2; ++i) {
    assert((int)res[i].size() <= max_len);
  }
  return res;
}

vector< string > gen_cut(int avg_cut_len, string necklace, int max_len) {
  uniform_int_distribution< int > len_distr(1, avg_cut_len - 1);
  uniform_int_distribution< int > loc_distr(0, (int)necklace.size() - 1);
  vector< string > res(2);
  while (true) {
    int len = len_distr(gen) + len_distr(gen);
    if ((int)res[0].size() + len > max_len) break;
    int loc = loc_distr(gen);
    uniform_int_distribution< int > rot_distr(0, len - 1);
    for (int i = 0; i < 2; ++i) {
      int rot = rot_distr(gen);
      int dir = gen() & 1;
      for (int j = 0; j < len; ++j) {
        int idx = rot;
        if (dir) {
          idx -= j;
        } else {
          idx += j;
        }
        idx = (idx % len + len) % len;
        idx = (loc + idx) % (int)necklace.size();
        res[i].push_back(necklace[idx]);
      }
    }
  }
  return res;
}

vector< string > gen_cut_hard(int avg_cut_len, int spec_cut_len, string necklace, int max_len) {
  uniform_int_distribution< int > len_distr(1, avg_cut_len - 1);
  uniform_int_distribution< int > loc_distr(0, (int)necklace.size() - 1);
  vector< string > res(2);
  bool spec_used = false;
  int spec_pos = max_len/4+gen()%(max_len/2);
  while (true) {
    int len = len_distr(gen) + len_distr(gen);
    if(!spec_used && (int)res[0].size() >= spec_pos){
      len = spec_cut_len;
      spec_used = true;
    }
    if ((int)res[0].size() + len > max_len) break;
    int loc = loc_distr(gen);
    uniform_int_distribution< int > rot_distr(0, len - 1);
    for (int i = 0; i < 2; ++i) {
      int rot = rot_distr(gen);
      int dir = gen() & 1;
      int add_end = gen() &1;
      for (int j = 0; j < len; ++j) {
        int idx = rot;
        if (dir) {
          idx -= j;
        } else {
          idx += j;
        }
        idx = (idx % len + len) % len;
        idx = (loc + idx) % (int)necklace.size();
        if(add_end){
          res[i].push_back(necklace[idx]);
        } else {
          res[i].push_back(necklace[idx]);
        }
      }
    }
  }
  return res;
}

int test_num = 0;

int group_start_test_num;
int test_group = 0;

vector< int > group_lim;

bool write_test(vector< string > inp, bool allow_flip = true) {
  for (int i = 0; i < (int)inp.size(); ++i) {
    if (allow_flip && (gen() & 1)) {
      reverse(inp[i].begin(), inp[i].end());
    }
  }
  ofstream fout("input/input" + to_string(test_num) + ".txt");
  for (int i = 0; i < (int)inp.size(); ++i) {
    assert((int)inp[i].size() <= group_lim[test_group]);
    fout << inp[i] << '\n';
  }
  fout.close();
  system(("./gen/sol <input/input" + to_string(test_num) + ".txt >gen/sol.out").c_str());
  system(("./gen/sol_substr <input/input" + to_string(test_num) + ".txt >gen/sol_substr.out").c_str());
  ifstream fin("gen/sol.out");
  int sol_res;
  fin>>sol_res;
  fin.close();
  fin.open("gen/sol_substr.out");
  int sol_substr_res;
  fin>>sol_substr_res;
  fin.close();
  if(sol_res == sol_substr_res)
    return false;
  cerr<<sol_res<<endl;
  cerr<<sol_substr_res<<endl;
  cerr<<endl;
  GEN_file << test_num << "\n";
  desc_file << test_num << "\t" << test_num-group_start_test_num+1;
  ++test_num;
  return true;
}

string rand_str(int len) {
  string res;
  for (int i = 0; i < len; ++i) {
    res.push_back('a' + gen() % 26);
  }
  return res;
}

vector< string > rand_str_set(int len, int cnt) {
  vector< string > res;
  for (int i = 0; i < cnt; ++i) {
    bool valid = false;
    string cand;
    while (!valid) {
      cand = rand_str(len);
      valid = true;
      for (int j = 0; j < len - 1; ++j) {
        if (cand[j] == cand[j + 1]) valid = false;
      }
      for (int t = 0; t < 2; ++t) {
        for (int j = 0; j < (int)res.size(); ++j) {
          for (int k = 0; k < len; ++k) {
            for (int l = 0; l < len; ++l) {
              for (int m = 0; m + k < len && m + l < len; ++m) {
                if (m > len / 2) valid = false;
                if (res[j][k + m] != cand[l + m]) break;
              }
            }
          }
        }
        reverse(cand.begin(), cand.end());
      }
    }
    res.push_back(cand);
  }
  return res;
}

vector< string > rand_perm_set(int len, int cnt) {
  string cand = rand_str(len);
  vector< string > res;
  for (int i = 0; i < cnt; ++i) {
    bool valid = false;
    while (!valid) {
      shuffle(cand.begin(), cand.end(), gen);
      valid = true;
      for (int j = 0; j < len - 1; ++j) {
        if (cand[j] == cand[j - 1]) valid = false;
      }
      for (int t = 0; t < 2; ++t) {
        for (int j = 0; j < (int)res.size(); ++j) {
          for (int k = 0; k < len; ++k) {
            for (int l = 0; l < len; ++l) {
              for (int m = 0; m + k < len && m + l < len; ++m) {
                if (m > len / 2) valid = false;
                if (res[j][k + m] != cand[l + m]) break;
              }
            }
          }
        }
        reverse(cand.begin(), cand.end());
      }
    }
    res.push_back(cand);
  }
  return res;
}

int main(int argc, char **argv) {
  if (argc != 3) {
    cerr << "usage: gen GEN desc_file\n";
    return 0;
  }
  // ll seed = static_cast<ll> (std::chrono::system_clock::now().time_since_epoch().count());
  ll seed = 1554894215733595215;
  gen = mt19937_64(seed);
  gen.discard(gen.state_size);

  GEN_file = ofstream(argv[1]);
  desc_file = ofstream(argv[2]);

  group_lim = {7, 100, 400, 3000, 3000};
  vector< int > group_points = {0, 25, 20, 40, 15};

  cout << "Sample group start = " << test_num << '\n';
  GEN_file << "# ST: " << group_points[test_group] << "\n";
  GEN_file << "# examples\n";
  desc_file << "Examples\n";
  {
    vector< string > s = {"zxyabcd", "yxbadcz"};
    bool suc;
    do{
      suc = write_test(s, false);
    }while(!suc);
    desc_file << ": example with reversal\n";
  }

  for (test_group = 1; test_group < (int)group_lim.size(); ++test_group) {
    group_start_test_num = test_num;
    cout << "Test group start = " << test_num << '\n';
    GEN_file << "# ST: " << group_points[test_group] << "\n";
    desc_file << "Test group "<<test_group<<": n = "<<group_lim[test_group]<<"\n";
    {
      bool suc;
      do{
        string atom = rand_str(1);
        vector< string > s(2);
        string common = rand_str(gen() % (group_lim[test_group] / 9));
        for (int i = 0; i < 2; ++i) {
          string l = rand_str(gen() % (group_lim[test_group] / 9));
          string r = rand_str(gen() % (group_lim[test_group] / 9));
          if(!i){
            l += common;
          } else {
            r = common+r;
          }
          string m;
          while ((int)(l.size() + m.size() + atom.size() + r.size()) <= group_lim[test_group]) {
            m += atom;
          }
          s[i] = l + m + r;
        }
        suc = write_test(s);
      }while(!suc);
      desc_file << ": equal letter repeat in middle\n";
    }
    {
      bool suc;
      do{
        string atom;
        do {
          atom = rand_str(2);
        } while (atom[0] == atom[1]);
        vector< string > s(2);
        string common = rand_str(gen() % (group_lim[test_group] / 9));
        for (int i = 0; i < 2; ++i) {
          string l = rand_str(gen() % (group_lim[test_group] / 9));
          string r = rand_str(gen() % (group_lim[test_group] / 9));
          if(!i){
            l += common;
          } else {
            r = common+r;
          }
          string m;
          while ((int)(l.size() + m.size() + atom.size() + r.size()) <= group_lim[test_group]) {
            m += atom;
          }
          s[i] = l + m + r;
        }
        suc = write_test(s);
      }while(!suc);
      desc_file << ": equal two-letter atom repeat in middle\n";
    }
    {
      int avg_cnt = 2 * sqrt(group_lim[test_group]);
      bool suc;
      do{
        string atom = rand_str(1);
        if(test_group >= 3)
          avg_cnt *= 1.5;
        vector< string > tokens = rand_str_set(4, 10);
        vector< string > s =
          gen_hard(atom, avg_cnt, tokens, group_lim[test_group]);
        suc = write_test(s);
      }while(!suc);
      desc_file << ": gen-hard: 10 tokens of len 4, single letter atom, avg atom cnt = "<<avg_cnt<<"\n";
    }
    {
      int avg_cnt = 1 * sqrt(group_lim[test_group]);
      bool suc;
      do{
        string atom;
        do {
          atom = rand_str(2);
        } while (atom[0] == atom[1]);
        if(test_group >= 3)
          avg_cnt *= 1.5;
        vector< string > tokens = rand_str_set(4, 10);
        vector< string > s =
          gen_hard(atom, avg_cnt, tokens, group_lim[test_group]);
        suc = write_test(s);
      }while(!suc);
      desc_file << ": gen-hard: 10 tokens of len 4, dual letter atom, avg atom cnt = "<<avg_cnt<<"\n";
    }
    if (test_group >= 2) {
      int neck_len = group_lim[test_group]/5;
      int avg_len = neck_len/5; 
      bool suc;
      do{
        string necklace = rand_str(neck_len);
        vector< string > s = gen_cut_hard(avg_len, avg_len*4, necklace, group_lim[test_group]);
        suc = write_test(s);
      }while(!suc);
      desc_file << ": gen-cut-hard: necklace len = "<<neck_len<<", avg cut len = "<<avg_len<<"\n";
    }
    if (test_group >= 2) {
      int neck_len = max(group_lim[test_group]/15, 40);
      int avg_len = min(neck_len/5, 10);
      bool suc;
      do{
        string necklace = rand_str(neck_len);
        vector< string > s = gen_cut_hard(avg_len, avg_len*4, necklace, group_lim[test_group]);
        suc = write_test(s);
      }while(!suc);
      desc_file << ": gen-cut-hard: necklace len = "<<neck_len<<", avg cut len = "<<avg_len<<"\n";
    } else {
      bool suc;
      do{
        string necklace = rand_str(group_lim[test_group]);
        vector< string > s =
          gen_cut(sqrt(group_lim[test_group]) / 3, necklace, group_lim[test_group]);
        suc = write_test(s);
      }while(!suc);
      desc_file << ": gen-cut: group-len necklace, avg cut len = "<<sqrt(group_lim[test_group]) / 3<<"\n";
    }
    if (test_group >= 3) {
      int n = group_lim[test_group] - 1024 - 3*550 - 4*7;
      bool suc;
      do{
        assert(n > 0);
        char spec_char = rand_str(1)[0];
        string a(550, spec_char);
        string b = rand_str(n);
        vector< string > rstr = rand_perm_set(7, 4);
        string ah = "ah";
        if (test_group > 3) {
          ++ah[0];
          ++ah[1];
        }
        string anti_hash[2];
        for (int i = 0; i < 1024; ++i) {
          anti_hash[0].push_back(ah[__builtin_popcount(i) & 1]);
          anti_hash[1].push_back(ah[!(__builtin_popcount(i) & 1)]);
        }

        vector< string > s = {b + a + a + rstr[0] + anti_hash[0] + rstr[1] + a,
          a + rstr[0] + anti_hash[1] + rstr[1] + a + a + b};
        /*
           vector< string > s = {b + c + rstr[2] + a + rstr[0] + anti_hash[0] + rstr[1] + aa,
           aa + rstr[0] + anti_hash[1] + rstr[1] + a + rstr[3] + c + b};
           */
        suc = write_test(s);
      }while(!suc);
      desc_file << ": anti hash mod 2^64, part len = "<<n<<"\n";
    }
    if (test_group >= 2) {
      for (int i = 1; i < 3; ++i) {
        int cnt_segs = 2+i;
        bool suc;
        bool broken = false;
        do{
          if (test_group == 2 && cnt_segs != 3){
            broken = true;
            break;
          }
          int max_lr_len = 50;
          if(test_group == 2)
            max_lr_len = 10;
          int f_l_len = gen()%max_lr_len;
          int f_r_len = gen()%max_lr_len;
          int s_l_len = gen()%max_lr_len;
          int s_r_len = gen()%max_lr_len;
          int common_len = group_lim[test_group]-max(f_l_len+f_r_len, s_l_len+s_r_len);
          int n = common_len / (2*(1+cnt_segs));
          assert(n > 0);
          string a(2 * n - 7, rand_str(1)[0]);
          int shift = 1 + gen() % n + gen() % n;
          if (i > 0 && i < 3) {
            shift = 5 + gen() % 10;
          }
          assert((int)shift > 0 && shift < 2 * n);
          string b = rand_str(shift);
          string c = rand_str(2 * n - shift);
          vector< string > rstr = rand_perm_set(7, 8);
          vector< string > s(2);
          s[0] += b + c;
          s[1] += a;
          for(int j=0; j<cnt_segs-1; ++j){
            s[0] += rstr[2+2*j]+a;
            s[1] += rstr[2+2*j+1]+a;
          }
          s[0] += rstr[0] + a;
          s[1] += rstr[1] + c + b;
          /*
           * {b + c + rstr[0] + a + rstr[1] + a + rstr[2] + a + rstr[3] + a,
           a + rstr[4] + a + rstr[5] + a + rstr[6] + a + rstr[7] + c + b};
           */
          suc = write_test(s);
        }while(!suc);
        if(!broken)
          desc_file << ": anti LCS extend, cnt_segs = "<<cnt_segs<<"\n";
      }
    }
  }
  GEN_file.close();
  desc_file.close();
}
